/* TileRasterCache, a library to download tiles from several sources.
 *
 * Copyright (C) 2010 Prodevelop SL.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *  
 *   Prodevelop Integraci�n de Tecnolog�as SL
 *   Plaza don Juan de Vilarrasa , 14-5
 *   46000 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *   
 *   author: Alberto Romeu Carrasco (aromeu@prodevelop.es)
 */

package es.prodevelop.tilecache;

import es.prodevelop.gvsig.mini.common.IContext;
import es.prodevelop.gvsig.mini.common.impl.BaseEvent;
import es.prodevelop.gvsig.mini.exceptions.BaseException;
import es.prodevelop.gvsig.mini.geom.Extent;
import es.prodevelop.gvsig.mini.utiles.Cancellable;
import es.prodevelop.tilecache.generator.IBufferStrategy;
import es.prodevelop.tilecache.generator.ITileBufferIntersector;
import es.prodevelop.tilecache.generator.ITileSorter;
import es.prodevelop.tilecache.generator.TileGenerator;
import es.prodevelop.tilecache.provider.filesystem.strategy.ITileFileSystemStrategy;
import es.prodevelop.tilecache.renderer.MapRenderer;

/**
 * BackGround task to automatically download tiles. It should be executed on a
 * Thread
 * 
 * @author aromeu
 * 
 */
public class TileDownloaderTask implements Runnable {

	private TileGenerator generator;
	private IDownloadWaiter downloadWaiter;
	private int fromZoomLevel = 0;
	private int toZoomLevel = 0;
	private Cancellable cancellable;

	/**
	 * The constructor
	 * 
	 * @param context
	 *            The IContext implementation
	 * @param aMapRenderer
	 *            A valid MapRenderer @see Layers
	 * @param fromZoomLevel
	 *            minimum zoom level from where download tiles
	 * @param toZoomLevel
	 *            maximum zoom level to where download tiles
	 * @param cancellable
	 *            A Cancellable instance. On each iteration the Cancellable
	 *            state is checked to see if the process has to be stopped
	 * @param extent
	 *            The extent to be downloaded
	 * @param sorter
	 *            An ITileSorter implementation. If this method is not used to
	 *            feed a map viewer, this parameter can(must) be null
	 * @param callBackHandler
	 *            An IDownloadCallBackHandler implementation. TileProvider
	 *            should notify this handler that internally will notify an
	 *            IDownloadWaiter implementation
	 * @param intersector
	 *            An ITileBufferIntersector. This parama allows the library to
	 *            download only the tiles that intersects with a single polygon.
	 *            If the library is used to feed a map viewer where the area to
	 *            download is always an Extent this paramater can be null
	 * @param mode
	 *            MODE_ONLINE = 0 - Gets tiles from file system and from the
	 *            server MODE_OFFLINE = 1 - Only gets tiles that are on file
	 *            system MODE_UPDATE = 2 - If a tile is on the file system,
	 *            deletes it and tries to download MODE_DELETE = 3 - Deletes the
	 *            tile cache MODE_CONNECTED = 4 - Don't save any tile, always
	 *            download them
	 * @param fsStrategy
	 *            The ITileFileSystemStrategy
	 * @param bufferStrategy
	 *            The IBufferStrategy
	* @param notLocalCache if false not stores tiles in memory cache
	 * @throws BaseException
	 *             A basic check of the properties is done. If zoom levels are
	 *             invalid or the area to download is zero, then a BaseException
	 *             is thrown
	 */
	public TileDownloaderTask(IContext context, MapRenderer aMapRenderer,
			int fromZoomLevel, int toZoomLevel, Cancellable cancellable,
			Extent extent, ITileSorter sorter,
			IDownloadCallbackHandler callBackHandler,
			ITileBufferIntersector intersector, int mode,
			ITileFileSystemStrategy fsStrategy, IBufferStrategy bufferStrategy, boolean notLocalCache)
			throws BaseException {
		generator = new TileGenerator(context, aMapRenderer, fromZoomLevel,
				toZoomLevel, cancellable, extent, sorter, callBackHandler,
				intersector, mode, fsStrategy, bufferStrategy, notLocalCache);
		this.downloadWaiter = callBackHandler.getDownloadWaiter();
		this.fromZoomLevel = fromZoomLevel;
		this.toZoomLevel = toZoomLevel;
		this.cancellable = cancellable;
	}

	/**
	 * Runs the download process
	 */
	public void run() {
		try {
			downloadWaiter.onStartDownload();
			for (int i = fromZoomLevel; i <= toZoomLevel; i++) {
				generator.downloadLevel(i, cancellable);
			}
		} catch (Exception e) {
			downloadWaiter.onFatalError(new BaseEvent(
					"Fatal Error while executing TileDownloaderTask: "
							+ e.getMessage()));
		}
	}
}
